#include "base.h"
#include "delta.h"
#include "lognormal.h"

#include <stdexcept>

namespace times {
    BaseDistribution::BaseDistribution(std::string name, const double tau, std::mt19937_64 &rng) : m_name(std::move(name)), m_tau(tau), m_tau_jump(tau), m_rng(rng) {}

    BaseDistribution::BaseDistribution(std::string name, std::mt19937_64 &rng) : m_name(std::move(name)), m_rng(rng) {}

    double BaseDistribution::tau_wait() const {
        return std::exponential_distribution(1./m_tau_jump)(m_rng);
    }

    void BaseDistribution::setParameters(const std::unordered_map<std::string, double> &parameters) {
        m_tau = parameters.at("tau");
    }

    std::unordered_map<std::string, double> BaseDistribution::getParameters() const {
        return std::unordered_map<std::string, double>{
            {"tau", m_tau},
        };
    }


    BaseDistribution* BaseDistribution::createFromInput(const std::string& input, std::mt19937_64& rng) {
        if (input == "Delta")
            return new DeltaDistribution(rng);

        if (input == "LogNormal")
            return new LogNormalDistribution(rng);

        throw std::invalid_argument("Invalid input " + input);
    }
}